local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )
local mathutil = include( "modules/mathutil" )

---------------------------------------------------------
-- Local functions

function isCoreItem( unit )
		if unit:getTraits().largenano or unit:getTraits().bigshopcat or unit:getTraits().vip or unit:getTraits().public_term then
			return true
		elseif unit:hasAbility( "open_detention_cells" ) or unit:hasAbility( "hostage_rescuable" )  or unit:hasAbility( "open_security_boxes" ) then
			return true
		elseif unit:hasAbility( "useAugmentMachine" ) and not unit:getTraits().drill then
			return true
		end
	
	--traits().largenano
	--
end


function closestByWalking ( sim, units, userUnit, maxMP, fn )
	--simquery.findPath( sim, unit, startcell, endcell, maxMP, goalFn )
	
	local userLoc = userUnit:getLocation()
	local userLocx, userLocy = userUnit:getLocation()
	local userCell = sim:getCell( userLoc )
	--local nearestEmptyUserCell = simquery.findNearestEmptyCell( sim, userCell.x, userCell.y, userUnit )
	
	
	
--	local startCell = sim:getCell(self.unit:getLocation() )
	--local guardCells = sim:getCells("guard_spawn")

	--to do: reduce the cells we path to down to just the closest ones for each guard elevator

	local astar = include ("modules/astar" )
	local astar_handlers = include("sim/astar_handlers")
	local pather = astar.AStar:new(astar_handlers.aihandler:new(userUnit) ) -- self.unit) )
	local closestCell, closestPathDist, closestUnit
	for i, unit in ipairs(units) do
	--	log:write("checking dist-")
	--	log:write(unit:getUnitData().name)
		local x1, y1 = unit:getLocation()
		local unitCell = sim:getCell( x1, y1 )
		local nearestEmptyCell = simquery.findNearestEmptyCell( sim, x1, y1, userUnit )
		local nearestEmptyUserCell = simquery.findNearestEmptyCell( sim, userLocx, userLocy, unit )
		local canPath, pathCost = simquery.findPath( sim, userUnit, nearestEmptyUserCell, nearestEmptyCell )

		if pathCost then
			if not closestCell or pathCost < closestPathDist then
				closestCell = unitCell
				closestUnit = unit
				closestPathDist = pathCost
			end
		end
	end

	if closestUnit then
		log:write("closestIs-")
		log:write(closestUnit:getUnitData().name)
	end
		return closestUnit --{x=closestCell.x, y=closestCell.y}
end

function isUsefulItem( unit )
	if unit:getTraits().mainframe_status ~= "off" and not unit:getTraits().hasBeenPatrolChecked then
		if unit:getTraits().revealUnits == "mainframe_console" or unit:getTraits().revealUnits == "mainframe_camera"  or  unit:getTraits().showOutline or  unit:getTraits().revealDaemons then
			return true
		elseif unit:getTraits().router or unit:getTraits().power_core or unit:getTraits().tinker_anim  then
			return true
	--	elseif unit:getTraits().storeType == "standard" or unit:getTraits().storeType == "miniserver" then
	--		return true
	--	elseif unit:getTraits().powerGrid then
		elseif unit:getTraits().laser_gen or unit:getTraits().multiLockSwitch then
			return true
		end
	end
	
	--traits().largenano
	--
end

local decker_followGuard = 
	{
		name = STRINGS.FLAVORED.ITEMS.AUGMENTS.FOLLOWPATROL,

		createToolTip = function( self, sim, abilityOwner, abilityUser, targetID )
			return abilityutil.formatToolTip(STRINGS.FLAVORED.ITEMS.AUGMENTS.FOLLOWPATROL, STRINGS.FLAVORED.ITEMS.AUGMENTS.FOLLOWPATROL_DESC)
		end,
		
		onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
			local tooltip = util.tooltip( hud._screen )
			local section = tooltip:addSection()
			local canUse, reason = abilityUser:canUseAbility( sim, self, abilityOwner, targetUnitID )		
			local targetUnit = sim:getUnit( targetUnitID )
	        section:addLine( STRINGS.FLAVORED.ITEMS.AUGMENTS.FOLLOWPATROL )
			if targetUnit then
					section:addLine(STRINGS.FLAVORED.ITEMS.AUGMENTS.FOLLOWPATROL_DESC, abilityOwner:getTraits().advPWR )
			end
			if reason then
				section:addRequirement( reason )
			end
			return tooltip
		end,
		
		proxy = true,

		profile_icon = "gui/icons/skills_icons/skills_icon_small/icon-item_overwatch_small.png",
		getName = function( self, sim, unit )
			return self.name
		end,

        canUseWhileDragging = true,

		isTarget = function( self, sim, userUnit, targetUnit )
			if simquery.isEnemyTarget( userUnit:getPlayerOwner(), targetUnit ) and targetUnit:getTraits().isAgent and not targetUnit:getTraits().isDrone then
			local x0, y0 = userUnit:getLocation()
				if targetUnit:getBrain():getSituation() == sim:getNPC():getIdleSituation() and not userUnit:getTraits().hasUsedFollowGuard 
				and not ( targetUnit:isKO() or targetUnit:isDead() or  targetUnit:getTraits().noObserve ) and targetUnit:getBrain():getSituation() == sim:getNPC():getIdleSituation() then
					return true
				end
			end
					
			return false
		end,

		acquireTargets = function( self, targets, game, sim, unit, userUnit )
			local units = {}
			local x0, y0 = userUnit:getLocation()
			for _, targetUnit in pairs(sim:getAllUnits()) do
                if self:isTarget( sim, userUnit, targetUnit ) then 

    				if sim:canUnitSeeUnit( userUnit, targetUnit ) then
						table.insert( units, targetUnit )
					else 
						for _, cameraUnit in pairs(sim:getAllUnits()) do 
							if cameraUnit:getTraits().peekID == userUnit:getID() and sim:canUnitSeeUnit( cameraUnit, targetUnit ) then -- peek thru doors
								table.insert( units, targetUnit )
                               break
							end
						end
					end
				end
			end

			return targets.unitTarget( game, units, self, unit, userUnit )
		end,

		canUseAbility = function( self, sim, unit, userUnit, targetID )
			local targetUnit = sim:getUnit( targetID )
			if targetUnit then 
				local x0, y0 = userUnit:getLocation()
			end

			return true 
		end, 
		
		onSpawnAbility = function( self, sim, unit )
			self.abilityOwner = unit
			self.patrollingGuard = nil
			self.targetedCell = nil
			self.targetedUnit = nil
			self.patrolGuardBase = nil
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self )
		end,
			
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
			self.patrollingGuard = nil
			self.abilityOwner = nil
			self.targetedCell = nil
			self.targetedUnit = nil
			self.patrolGuardBase = nil
		end,
		
		-- trigger - on reach patrolLoc, 'patrolReached', maybe play tinker anim
		-- recreate table with fn excluding 'patrolReached'
		-- if 'noted' not 'reached', recreate table
		-- else find objective and make patrol (nearest reachable x3?)
		
		onTrigger = function ( self, sim, evType, evData )  -- need to get this working

			if evType == simdefs.TRG_UNIT_WARP then
				local patrollingGuard = self.patrollingGuard
				local abilityOwner = self.abilityOwner
				local targetedCell = self.targetedCell
				local targetedUnit = self.targetedUnit
				local patrollingGuardBase = self.patrolGuardBase

				if targetedCell and evData.to_cell and evData.unit == patrollingGuard and evData.to_cell == targetedCell then
					log:write("REACHED!")
					targetedUnit:getTraits().hasBeenPatrolChecked = true
					self:doSetPatrol( sim, abilityOwner, abilityOwner, patrollingGuardBase )
				end
			end
		end,
		
		executeAbility = function( self, sim, unit, userUnit, target )
			self.patrolGuardBase = target
			self:doSetPatrol( sim, unit, userUnit, target )
			userUnit:getTraits().hasUsedFollowGuard = true
		end,
		

		doSetPatrol = function( self, sim, unit, userUnit, target )
			local target = sim:getUnit(target)
			local x0, y0 = userUnit:getLocation()
			local x1, y1 = target:getLocation()
			local player = sim:getPC()
			self.patrollingGuard = target
			
			local usefulItems = {}
			for _, unit in pairs( sim:getAllUnits() ) do
				if isUsefulItem(unit) then
					if not unit:getTraits().hasBeenPatrolChecked then
						table.insert( usefulItems, unit )
					end
				end
			end
			
			local coreItems = {}
			
			for _, unit in pairs( sim:getAllUnits() ) do
				if isCoreItem(unit) then
					if not unit:getTraits().hasBeenPatrolChecked then
						table.insert( coreItems, unit )
					end
				end
			end
			
			local patrolLoc
			local closestUsefulItem = nil
			
			if #usefulItems > 0 then
				closestUsefulItem = closestByWalking ( sim, usefulItems, target )
				local x9, y9 = closestUsefulItem:getLocation()
				patrolLoc = simquery.findNearestEmptyCell( sim, x9, y9, target )
			elseif #coreItems > 0 then
				closestCoreItem = closestByWalking ( sim, coreItems, target )
				local x9, y9 = closestCoreItem:getLocation()
				patrolLoc = simquery.findNearestEmptyCell( sim, x9, y9, target )
			end
			
			if patrolLoc then
				if closestUsefulItem then
					local x9, y9 = closestUsefulItem:getLocation()
					local x8, y8 = patrolLoc.x, patrolLoc.y
				--	sim:getNPC():getIdleSituation():generatePatrolPath( target, x8, y8 )
				--	sim:getNPC():getIdleSituation():generateStationaryPath( target, x8, y8 )
					local newFacing = simquery.getDirectionFromDelta(x9-x8,y9-y8)
					target:getTraits().patrolPath = { { x = x8, y = y8, facing = newFacing } }
					target:getBrain():reset()
					self.targetedUnit = closestUsefulItem
					self.targetedCell =  sim:getCell( x8, y8 )
				elseif target:getBrain():getSituation() == sim:getNPC():getIdleSituation() and closestCoreItem then
					local x9, y9 = closestCoreItem:getLocation()
					local x8, y8 = patrolLoc.x, patrolLoc.y
					sim:getNPC():getIdleSituation():generatePatrolPath( target, x8, y8 )
				end
			end
		end
	}
return decker_followGuard